<?php

namespace Drupal\cloud\Commands;

use Drupal\cloud\Service\CloudServiceInterface;
use Drupal\Component\Serialization\Yaml;
use Drupal\Component\Utility\Random;
use Drupal\Core\Extension\ExtensionPathResolver;
use Drush\Commands\DrushCommands;

/**
 * A Drush Behat commands file.
 */
class BehatCommands extends DrushCommands implements BehatCommandsInterface {

  /**
   * Cloud Service.
   *
   * @var \Drupal\cloud\Service\CloudServiceInterface
   */
  protected $cloudService;

  /**
   * The extension path resolver.
   *
   * @var \Drupal\Core\Extension\ExtensionPathResolver
   */
  protected $extensionPathResolver;

  /**
   * BehatCommands constructor.
   *
   * @param \Drupal\cloud\Service\CloudServiceInterface $cloud_service
   *   Cloud Service.
   * @param \Drupal\Core\Extension\ExtensionPathResolver $extension_path_resolver
   *   The extension path resolver.
   */
  public function __construct(CloudServiceInterface $cloud_service, ExtensionPathResolver $extension_path_resolver) {
    $this->cloudService = $cloud_service;
    $this->extensionPathResolver = $extension_path_resolver;
    parent::__construct();
  }

  /**
   * Preprocess behat test feature files.
   *
   * @param array $options
   *   An associative array of options whose values come from cli, config, etc.
   *
   * @option string $behat_path
   *   The behat relative path from the module root directory. Other files are
   *   relative to this path.
   * @option string $params_file
   *   The file defining parameters for templates.
   * @option string $templates_dir
   *   The directory where the feature file templates exist.
   * @option string $output_dir
   *   The directory where this preprocessor creates files.
   *
   * @command cloud:behat:preprocess
   * @aliases cloud-behat-pre
   */
  public function preprocess(array $options = [
    'behat_path' => self::BEHAT_PATH,
    'params_file' => 'behat_params.yml',
    'templates_dir' => 'features/templates',
    'output_dir' => 'features/tmp',
  ]): void {

    $base_path = $this->extensionPathResolver->getPath('module', 'cloud') . '/' . $options['behat_path'] . '/';

    $opt_params_file = $options['params_file'];
    $params_file = $base_path . $opt_params_file;

    $opt_templates_dir = $options['templates_dir'];
    $templates_dir = $base_path . $opt_templates_dir;

    $opt_output_dir = $options['output_dir'];
    $output_dir = $base_path . $opt_output_dir;

    if (!is_readable($params_file)) {
      $this->logger()->warning(dt('No such file exists or is readable for PARAMS_FILE: @opt_params_file', [
        '@opt_params_file' => $opt_params_file,
      ]));
      return;
    }

    if (!is_readable($templates_dir) || !is_dir($templates_dir)) {
      $this->logger()->warning(dt('No such directory exists or is readable for TEMPLATES_DIR: @opt_templates_dir', [
        '@opt_templates_dir' => $opt_templates_dir,
      ]));
      return;
    }

    $params = Yaml::decode(file_get_contents($params_file));
    if (empty($params)) {
      $this->logger()->warning(dt('No valid parameters are set in PARAMS_FILE: @opt_params_file', [
        '@opt_params_file' => $opt_params_file,
      ]));
      return;
    }
    $this->randomizeName($params);

    if (!file_exists($output_dir)) {
      if (!mkdir($tmp_dir = $output_dir, 0755) && !is_dir($tmp_dir)) {
        $this->logger()->error(dt('Failed to create a directory of OUTPUT_DIR: @opt_output_dir'), [
          '@opt_output_dir' => $opt_output_dir,
        ]);
        return;
      }
      $this->logger()->notice(dt('Created: @opt', ['@opt' => $opt_output_dir]));
    }
    if (!is_dir($output_dir) || !is_writable($output_dir)) {
      $this->logger()->error(dt('Failed to find a writable directory of OUTPUT_DIR: @opt_output_dir', [
        '@opt_output_dir' => $opt_output_dir,
      ]));
      return;
    }

    if ($handle = opendir($templates_dir)) {
      $file_count = count(scandir($templates_dir)) - 2;
      $this->logger()->notice(dt('Processing @file_count file(s) under @opt_templates_dir...', [
        '@file_count' => $file_count,
        '@opt_templates_dir' => $opt_templates_dir,
      ]));
      $i = 0;
      while ($file = readdir($handle)) {
        if ($file !== '.' && $file !== '..') {
          $output = $output_dir . '/' . $file;
          $input = $templates_dir . '/' . $file;
          file_put_contents($output, $this->cloudService->renderTemplate(file_get_contents($input), $params));
          $i++;
          $this->logger()->notice(dt(' [@index/@file_count] Created: @opt_output_dir/@file', [
            '@file' => $file,
            '@file_count' => $file_count,
            '@index' => $i,
            '@opt_output_dir' => $opt_output_dir,
          ]));
        }
      }
      closedir($handle);
      $this->logger()->success(dt('Finished processing the templates.'));
    }
  }

  /**
   * Replace a specified random string with a randomized name. Case sensitive.
   *
   * @param array $params
   *   An associative array of params passed by reference.
   */
  private function randomizeName(array &$params): void {
    $random = (new Random())->name(self::RANDOM_NAME_LENGTH);
    foreach ($params as $key => $value) {
      if (preg_match('/(^.*)(@random)(.*$)/i', $value, $matches)) {
        if ($matches[2] === '@random') {
          $rand = strtolower($random);
        }
        elseif ($matches[2] === '@RANDOM') {
          $rand = strtoupper($random);
        }
        else {
          $rand = $random;
        }
        $params[$key] = $matches[1] . $rand . $matches[3];
      }
    }
  }

}
